import requests

from server import *

OAUTH_BASE = 'https://oauth.campus.rwth-aachen.de/oauth2waitress/oauth2.svc/'
MOODLE_BASE = 'https://moped.ecampus.rwth-aachen.de/proxy/api/v2/eLearning/Moodle/'

def moodleget(endpoint, token, **args):
	args['token'] = token
	r = requests.request('GET', MOODLE_BASE+endpoint, params=args)
	return r.json()

def oauthget(endpoint, **args):
	args['client_id'] = config['L2P_APIKEY']
	r = requests.request('POST', OAUTH_BASE+endpoint, data=args)
	return r.json()

@app.route('/internal/l2pauth')
def start_l2pauth():
	return "L2P is no longer available."

@app.route('/internal/moodleauth')
def start_moodleauth():
	if 'L2P_APIKEY' not in config:
		return render_template("500.html"), 500
	code = oauthget('code', scope='moodle.rwth')
	session['oauthcode'] = code['device_code']
	session['oauthscope'] = 'moodle'
	return redirect(code['verification_url']+'?q=verify&d='+code['user_code'])

@app.route('/internal/moodlel2pauth')
def start_moodlel2pauth():
	return start_moodleauth()

@app.route('/internal/rwthauth')
def start_rwthauth():
	if 'L2P_APIKEY' not in config:
		return render_template("500.html"), 500
	code = oauthget('code', scope='userinfo.rwth')
	session['oauthcode'] = code['device_code']
	session['oauthscope'] = 'rwth'
	return redirect(code['verification_url']+'?q=verify&d='+code['user_code'])

@app.before_request
def finish_oauth():
	if 'L2P_APIKEY' not in config:
		return
	if 'oauthcode' not in session or 'oauthscope' not in session:
		return
	token = oauthget('token', code=session['oauthcode'], grant_type='device')
	if token.get('status') != 'ok':
		return
	del session['oauthcode']

	if session['oauthscope'] not in ['l2p', 'rwth', 'moodle', 'l2pandmoodle']:
		return
	session['rwthintern'] = True
	if session['oauthscope'] == 'moodle' or session['oauthscope'] == 'l2pandmoodle':
		data = moodleget('getmyenrolledcourses', token['access_token'])
		if data and data.get('Data'):
			session['moodle_courses'] = []
			for course in data['Data']:
				session['moodle_courses'].append(str(course['id']))
		else:
			notify_admins('endpoint_exception', traceback="finish_oauth failed while getting moodle courses, data={}".format(str(data)))
	del session['oauthscope']
	oauthget('token', refresh_token=token['refresh_token'], grant_type='invalidate')
